# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: voltha.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
import yang_options_pb2 as yang__options__pb2
import meta_pb2 as meta__pb2
import common_pb2 as common__pb2
import health_pb2 as health__pb2
import logical_device_pb2 as logical__device__pb2
import device_pb2 as device__pb2
import adapter_pb2 as adapter__pb2
import openflow_13_pb2 as openflow__13__pb2
try:
  yang__options__pb2 = openflow__13__pb2.yang__options__pb2
except AttributeError:
  yang__options__pb2 = openflow__13__pb2.yang_options_pb2
import omci_mib_db_pb2 as omci__mib__db__pb2
import omci_alarm_db_pb2 as omci__alarm__db__pb2

from meta_pb2 import *
from common_pb2 import *
from health_pb2 import *
from logical_device_pb2 import *
from device_pb2 import *
from adapter_pb2 import *
from openflow_13_pb2 import *

DESCRIPTOR = _descriptor.FileDescriptor(
  name='voltha.proto',
  package='voltha',
  syntax='proto3',
  serialized_pb=_b('\n\x0cvoltha.proto\x12\x06voltha\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x12yang_options.proto\x1a\nmeta.proto\x1a\x0c\x63ommon.proto\x1a\x0chealth.proto\x1a\x14logical_device.proto\x1a\x0c\x64\x65vice.proto\x1a\radapter.proto\x1a\x11openflow_13.proto\x1a\x11omci_mib_db.proto\x1a\x13omci_alarm_db.proto\"\x87\x01\n\x0b\x44\x65viceGroup\x12\x11\n\x02id\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12\x39\n\x0flogical_devices\x18\x02 \x03(\x0b\x32\x15.voltha.LogicalDeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12*\n\x07\x64\x65vices\x18\x03 \x03(\x0b\x32\x0e.voltha.DeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\"2\n\x0c\x44\x65viceGroups\x12\"\n\x05items\x18\x01 \x03(\x0b\x32\x13.voltha.DeviceGroup\"\x7f\n\x12\x41larmFilterRuleKey\"b\n\x12\x41larmFilterRuleKey\x12\x06\n\x02id\x10\x00\x12\x08\n\x04type\x10\x01\x12\x0c\n\x08severity\x10\x02\x12\x0f\n\x0bresource_id\x10\x03\x12\x0c\n\x08\x63\x61tegory\x10\x04\x12\r\n\tdevice_id\x10\x05:\x05\xf0\xf6\xcd\x1d\x00\"\\\n\x0f\x41larmFilterRule\x12:\n\x03key\x18\x01 \x01(\x0e\x32-.voltha.AlarmFilterRuleKey.AlarmFilterRuleKey\x12\r\n\x05value\x18\x02 \x01(\t\"H\n\x0b\x41larmFilter\x12\x11\n\x02id\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12&\n\x05rules\x18\x02 \x03(\x0b\x32\x17.voltha.AlarmFilterRule\"4\n\x0c\x41larmFilters\x12$\n\x07\x66ilters\x18\x01 \x03(\x0b\x32\x13.voltha.AlarmFilter\"I\n\x07Logging\x12(\n\x05level\x18\x01 \x01(\x0e\x32\x19.voltha.LogLevel.LogLevel\x12\x14\n\x0cpackage_name\x18\x02 \x01(\t\"^\n\x0c\x43oreInstance\x12\x1a\n\x0binstance_id\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12+\n\x06health\x18\x02 \x01(\x0b\x32\x14.voltha.HealthStatusB\x05\xe2\xf6\xcd\x1d\x00:\x05\xf8\xf6\xcd\x1d\x01\";\n\rCoreInstances\x12#\n\x05items\x18\x01 \x03(\x0b\x32\x14.voltha.CoreInstance:\x05\xf8\xf6\xcd\x1d\x01\"\xe8\x03\n\x06Voltha\x12\x16\n\x07version\x18\x01 \x01(\tB\x05\xe8\xf6\xcd\x1d\x01\x12,\n\x08\x61\x64\x61pters\x18\x02 \x03(\x0b\x32\x0f.voltha.AdapterB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x39\n\x0flogical_devices\x18\x03 \x03(\x0b\x32\x15.voltha.LogicalDeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12*\n\x07\x64\x65vices\x18\x04 \x03(\x0b\x32\x0e.voltha.DeviceB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x33\n\x0c\x64\x65vice_types\x18\x05 \x03(\x0b\x32\x12.voltha.DeviceTypeB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x35\n\rdevice_groups\x18\x06 \x03(\x0b\x32\x13.voltha.DeviceGroupB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12\x35\n\ralarm_filters\x18\x07 \x03(\x0b\x32\x13.voltha.AlarmFilterB\t\xe2\xf6\xcd\x1d\x04\n\x02id\x12@\n\x11omci_mib_database\x18\x1c \x03(\x0b\x32\x13.omci.MibDeviceDataB\x10\xe2\xf6\xcd\x1d\x0b\n\tdevice_id\x12\x45\n\x13omci_alarm_database\x18\x1d \x03(\x0b\x32\x16.alarm.AlarmDeviceDataB\x10\xe2\xf6\xcd\x1d\x0b\n\tdevice_id:\x05\xf8\xf6\xcd\x1d\x01\"\xa4\x01\n\x10SelfTestResponse\x12\x37\n\x06result\x18\x01 \x01(\x0e\x32\'.voltha.SelfTestResponse.SelfTestResult\"P\n\x0eSelfTestResult\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07\x46\x41ILURE\x10\x01\x12\x11\n\rNOT_SUPPORTED\x10\x02\x12\x11\n\rUNKNOWN_ERROR\x10\x03:\x05\xf0\xf6\xcd\x1d\x00\":\n\x11OfAgentSubscriber\x12\x12\n\nofagent_id\x18\x01 \x01(\t\x12\x11\n\tvoltha_id\x18\x02 \x01(\t2\xf0-\n\rVolthaService\x12O\n\x0eUpdateLogLevel\x12\x0f.voltha.Logging\x1a\x16.google.protobuf.Empty\"\x14\x82\xd3\xe4\x93\x02\x0e\x12\x0c/api/v1/logs\x12\x44\n\tGetVoltha\x12\x16.google.protobuf.Empty\x1a\x0e.voltha.Voltha\"\x0f\x82\xd3\xe4\x93\x02\t\x12\x07/api/v1\x12u\n\x11ListCoreInstances\x12\x16.google.protobuf.Empty\x1a\x15.voltha.CoreInstances\"1\x82\xd3\xe4\x93\x02\x13\x12\x11/api/v1/instances\x8a\xf7\xcd\x1d\x07\n\x05items\x8a\xf7\xcd\x1d\x07\x12\x05items\x12S\n\x0fGetCoreInstance\x12\n.voltha.ID\x1a\x14.voltha.CoreInstance\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/api/v1/instances/{id}\x12\x61\n\x0cListAdapters\x12\x16.google.protobuf.Empty\x1a\x10.voltha.Adapters\"\'\x82\xd3\xe4\x93\x02\x12\x12\x10/api/v1/adapters\x8a\xf7\xcd\x1d\n\n\x08\x61\x64\x61pters\x12{\n\x12ListLogicalDevices\x12\x16.google.protobuf.Empty\x1a\x16.voltha.LogicalDevices\"5\x82\xd3\xe4\x93\x02\x19\x12\x17/api/v1/logical_devices\x8a\xf7\xcd\x1d\x11\n\x0flogical_devices\x12[\n\x10GetLogicalDevice\x12\n.voltha.ID\x1a\x15.voltha.LogicalDevice\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/api/v1/logical_devices/{id}\x12r\n\x16ListLogicalDevicePorts\x12\n.voltha.ID\x1a\x14.voltha.LogicalPorts\"6\x82\xd3\xe4\x93\x02$\x12\"/api/v1/logical_devices/{id}/ports\x8a\xf7\xcd\x1d\x07\n\x05ports\x12\x83\x01\n\x14GetLogicalDevicePort\x12\x15.voltha.LogicalPortId\x1a\x13.voltha.LogicalPort\"?\x82\xd3\xe4\x93\x02.\x12,/api/v1/logical_devices/{id}/ports/{port_id}\x8a\xf7\xcd\x1d\x06\n\x04port\x12\x85\x01\n\x17\x45nableLogicalDevicePort\x12\x15.voltha.LogicalPortId\x1a\x16.google.protobuf.Empty\";\x82\xd3\xe4\x93\x02\x35\"3/api/v1/logical_devices/{id}/ports/{port_id}/enable\x12\x87\x01\n\x18\x44isableLogicalDevicePort\x12\x15.voltha.LogicalPortId\x1a\x16.google.protobuf.Empty\"<\x82\xd3\xe4\x93\x02\x36\"4/api/v1/logical_devices/{id}/ports/{port_id}/disable\x12|\n\x16ListLogicalDeviceFlows\x12\n.voltha.ID\x1a\x12.openflow_13.Flows\"B\x82\xd3\xe4\x93\x02$\x12\"/api/v1/logical_devices/{id}/flows\x8a\xf7\xcd\x1d\x07\n\x05\x66lows\x8a\xf7\xcd\x1d\x07\x12\x05items\x12\x83\x01\n\x1cUpdateLogicalDeviceFlowTable\x12\x1c.openflow_13.FlowTableUpdate\x1a\x16.google.protobuf.Empty\"-\x82\xd3\xe4\x93\x02\'\"\"/api/v1/logical_devices/{id}/flows:\x01*\x12\x84\x01\n\x1dUpdateLogicalDeviceMeterTable\x12\x1b.openflow_13.MeterModUpdate\x1a\x16.google.protobuf.Empty\".\x82\xd3\xe4\x93\x02(\"#/api/v1/logical_devices/{id}/meters:\x01*\x12{\n\x1cGetMeterStatsOfLogicalDevice\x12\n.voltha.ID\x1a\x1c.openflow_13.MeterStatsReply\"1\x82\xd3\xe4\x93\x02+\x12)/api/v1/logical_devices/{id}/meters_stats\x12\x92\x01\n\x1bListLogicalDeviceFlowGroups\x12\n.voltha.ID\x1a\x17.openflow_13.FlowGroups\"N\x82\xd3\xe4\x93\x02*\x12(/api/v1/logical_devices/{id}/flow_groups\x8a\xf7\xcd\x1d\r\n\x0b\x66low_groups\x8a\xf7\xcd\x1d\x07\x12\x05items\x12\x93\x01\n!UpdateLogicalDeviceFlowGroupTable\x12!.openflow_13.FlowGroupTableUpdate\x1a\x16.google.protobuf.Empty\"3\x82\xd3\xe4\x93\x02-\"(/api/v1/logical_devices/{id}/flow_groups:\x01*\x12]\n\x0bListDevices\x12\x16.google.protobuf.Empty\x1a\x0f.voltha.Devices\"%\x82\xd3\xe4\x93\x02\x11\x12\x0f/api/v1/devices\x8a\xf7\xcd\x1d\t\n\x07\x64\x65vices\x12\x64\n\rListDeviceIds\x12\x16.google.protobuf.Empty\x1a\x0b.voltha.IDs\".\x82\xd3\xe4\x93\x02\x13\x12\x11/api/v1/deviceids\x8a\xf7\xcd\x1d\x04\n\x02id\x8a\xf7\xcd\x1d\x07\x12\x05items\x12U\n\x10ReconcileDevices\x12\x0b.voltha.IDs\x1a\x16.google.protobuf.Empty\"\x1c\x82\xd3\xe4\x93\x02\x16\"\x11/api/v1/deviceids:\x01*\x12\x45\n\tGetDevice\x12\n.voltha.ID\x1a\x0e.voltha.Device\"\x1c\x82\xd3\xe4\x93\x02\x16\x12\x14/api/v1/devices/{id}\x12J\n\x0c\x43reateDevice\x12\x0e.voltha.Device\x1a\x0e.voltha.Device\"\x1a\x82\xd3\xe4\x93\x02\x14\"\x0f/api/v1/devices:\x01*\x12W\n\x0c\x45nableDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"#\x82\xd3\xe4\x93\x02\x1d\"\x1b/api/v1/devices/{id}/enable\x12Y\n\rDisableDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"$\x82\xd3\xe4\x93\x02\x1e\"\x1c/api/v1/devices/{id}/disable\x12W\n\x0cRebootDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"#\x82\xd3\xe4\x93\x02\x1d\"\x1b/api/v1/devices/{id}/reboot\x12W\n\x0c\x44\x65leteDevice\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"#\x82\xd3\xe4\x93\x02\x1d*\x1b/api/v1/devices/{id}/delete\x12u\n\rDownloadImage\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"6\x82\xd3\xe4\x93\x02\x30\"+/api/v1/devices/{id}/image_downloads/{name}:\x01*\x12\x82\x01\n\x16GetImageDownloadStatus\x12\x15.voltha.ImageDownload\x1a\x15.voltha.ImageDownload\":\x82\xd3\xe4\x93\x02\x34\x12\x32/api/v1/devices/{id}/image_downloads/{name}/status\x12u\n\x10GetImageDownload\x12\x15.voltha.ImageDownload\x1a\x15.voltha.ImageDownload\"3\x82\xd3\xe4\x93\x02-\x12+/api/v1/devices/{id}/image_downloads/{name}\x12\x66\n\x12ListImageDownloads\x12\n.voltha.ID\x1a\x16.voltha.ImageDownloads\",\x82\xd3\xe4\x93\x02&\x12$/api/v1/devices/{id}/image_downloads\x12x\n\x13\x43\x61ncelImageDownload\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"3\x82\xd3\xe4\x93\x02-*+/api/v1/devices/{id}/image_downloads/{name}\x12\x88\x01\n\x13\x41\x63tivateImageUpdate\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"C\x82\xd3\xe4\x93\x02=\"8/api/v1/devices/{id}/image_downloads/{name}/image_update:\x01*\x12\x86\x01\n\x11RevertImageUpdate\x12\x15.voltha.ImageDownload\x1a\x15.voltha.OperationResp\"C\x82\xd3\xe4\x93\x02=\"8/api/v1/devices/{id}/image_downloads/{name}/image_revert:\x01*\x12\\\n\x0fListDevicePorts\x12\n.voltha.ID\x1a\r.voltha.Ports\".\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/devices/{id}/ports\x8a\xf7\xcd\x1d\x07\n\x05ports\x12]\n\x13ListDevicePmConfigs\x12\n.voltha.ID\x1a\x11.voltha.PmConfigs\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/api/v1/devices/{id}/pm_configs\x12n\n\x15UpdateDevicePmConfigs\x12\x11.voltha.PmConfigs\x1a\x16.google.protobuf.Empty\"*\x82\xd3\xe4\x93\x02$\"\x1f/api/v1/devices/{id}/pm_configs:\x01*\x12m\n\x0fListDeviceFlows\x12\n.voltha.ID\x1a\x12.openflow_13.Flows\":\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/devices/{id}/flows\x8a\xf7\xcd\x1d\x07\n\x05\x66lows\x8a\xf7\xcd\x1d\x07\x12\x05items\x12\x83\x01\n\x14ListDeviceFlowGroups\x12\n.voltha.ID\x1a\x17.openflow_13.FlowGroups\"F\x82\xd3\xe4\x93\x02\"\x12 /api/v1/devices/{id}/flow_groups\x8a\xf7\xcd\x1d\r\n\x0b\x66low_groups\x8a\xf7\xcd\x1d\x07\x12\x05items\x12o\n\x0fListDeviceTypes\x12\x16.google.protobuf.Empty\x1a\x13.voltha.DeviceTypes\"/\x82\xd3\xe4\x93\x02\x16\x12\x14/api/v1/device_types\x8a\xf7\xcd\x1d\x0e\n\x0c\x64\x65vice_types\x12R\n\rGetDeviceType\x12\n.voltha.ID\x1a\x12.voltha.DeviceType\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/api/v1/device_types/{id}\x12s\n\x10ListDeviceGroups\x12\x16.google.protobuf.Empty\x1a\x14.voltha.DeviceGroups\"1\x82\xd3\xe4\x93\x02\x17\x12\x15/api/v1/device_groups\x8a\xf7\xcd\x1d\x0f\n\rdevice_groups\x12\x46\n\x10StreamPacketsOut\x12\x16.openflow_13.PacketOut\x1a\x16.google.protobuf.Empty\"\x00(\x01\x12\x45\n\x10ReceivePacketsIn\x12\x16.google.protobuf.Empty\x1a\x15.openflow_13.PacketIn\"\x00\x30\x01\x12K\n\x13ReceiveChangeEvents\x12\x16.google.protobuf.Empty\x1a\x18.openflow_13.ChangeEvent\"\x00\x30\x01\x12U\n\x0eGetDeviceGroup\x12\n.voltha.ID\x1a\x13.voltha.DeviceGroup\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/device_groups/{id}\x12_\n\x11\x43reateAlarmFilter\x12\x13.voltha.AlarmFilter\x1a\x13.voltha.AlarmFilter\" \x82\xd3\xe4\x93\x02\x1a\"\x15/api/v1/alarm_filters:\x01*\x12U\n\x0eGetAlarmFilter\x12\n.voltha.ID\x1a\x13.voltha.AlarmFilter\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/alarm_filters/{id}\x12\x64\n\x11UpdateAlarmFilter\x12\x13.voltha.AlarmFilter\x1a\x13.voltha.AlarmFilter\"%\x82\xd3\xe4\x93\x02\x1f\x1a\x1a/api/v1/alarm_filters/{id}:\x01*\x12[\n\x11\x44\x65leteAlarmFilter\x12\n.voltha.ID\x1a\x16.google.protobuf.Empty\"\"\x82\xd3\xe4\x93\x02\x1c*\x1a/api/v1/alarm_filters/{id}\x12_\n\x10ListAlarmFilters\x12\x16.google.protobuf.Empty\x1a\x14.voltha.AlarmFilters\"\x1d\x82\xd3\xe4\x93\x02\x17\x12\x15/api/v1/alarm_filters\x12L\n\tGetImages\x12\n.voltha.ID\x1a\x0e.voltha.Images\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/api/v1/devices/{id}/images\x12X\n\x08SelfTest\x12\n.voltha.ID\x1a\x18.voltha.SelfTestResponse\"&\x82\xd3\xe4\x93\x02 \"\x1e/api/v1/devices/{id}/self_test\x12V\n\x10GetMibDeviceData\x12\n.voltha.ID\x1a\x13.omci.MibDeviceData\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/api/v1/openomci/{id}/mib\x12]\n\x12GetAlarmDeviceData\x12\n.voltha.ID\x1a\x16.alarm.AlarmDeviceData\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/api/v1/openomci/{id}/alarm\x12s\n\rSimulateAlarm\x12\x1c.voltha.SimulateAlarmRequest\x1a\x15.voltha.OperationResp\"-\x82\xd3\xe4\x93\x02\'\"\"/api/v1/devices/{id}/simulate_larm:\x01*\x12\x43\n\tSubscribe\x12\x19.voltha.OfAgentSubscriber\x1a\x19.voltha.OfAgentSubscriber\"\x00\x42i\n\x13org.opencord.volthaB\x0cVolthaProtosZ+github.com/opencord/voltha-go/protos/voltha\xaa\x02\x16Opencord.Voltha.VolthaP\x03P\x04P\x05P\x06P\x07P\x08P\tb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,yang__options__pb2.DESCRIPTOR,meta__pb2.DESCRIPTOR,common__pb2.DESCRIPTOR,health__pb2.DESCRIPTOR,logical__device__pb2.DESCRIPTOR,device__pb2.DESCRIPTOR,adapter__pb2.DESCRIPTOR,openflow__13__pb2.DESCRIPTOR,omci__mib__db__pb2.DESCRIPTOR,omci__alarm__db__pb2.DESCRIPTOR,],
  public_dependencies=[meta__pb2.DESCRIPTOR,common__pb2.DESCRIPTOR,health__pb2.DESCRIPTOR,logical__device__pb2.DESCRIPTOR,device__pb2.DESCRIPTOR,adapter__pb2.DESCRIPTOR,openflow__13__pb2.DESCRIPTOR,])



_ALARMFILTERRULEKEY_ALARMFILTERRULEKEY = _descriptor.EnumDescriptor(
  name='AlarmFilterRuleKey',
  full_name='voltha.AlarmFilterRuleKey.AlarmFilterRuleKey',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='id', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='type', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='severity', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='resource_id', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='category', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='device_id', index=5, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=465,
  serialized_end=563,
)
_sym_db.RegisterEnumDescriptor(_ALARMFILTERRULEKEY_ALARMFILTERRULEKEY)

_SELFTESTRESPONSE_SELFTESTRESULT = _descriptor.EnumDescriptor(
  name='SelfTestResult',
  full_name='voltha.SelfTestResponse.SelfTestResult',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILURE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_SUPPORTED', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ERROR', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1595,
  serialized_end=1675,
)
_sym_db.RegisterEnumDescriptor(_SELFTESTRESPONSE_SELFTESTRESULT)


_DEVICEGROUP = _descriptor.Descriptor(
  name='DeviceGroup',
  full_name='voltha.DeviceGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.DeviceGroup.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='logical_devices', full_name='voltha.DeviceGroup.logical_devices', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='devices', full_name='voltha.DeviceGroup.devices', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=254,
  serialized_end=389,
)


_DEVICEGROUPS = _descriptor.Descriptor(
  name='DeviceGroups',
  full_name='voltha.DeviceGroups',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='voltha.DeviceGroups.items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=391,
  serialized_end=441,
)


_ALARMFILTERRULEKEY = _descriptor.Descriptor(
  name='AlarmFilterRuleKey',
  full_name='voltha.AlarmFilterRuleKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ALARMFILTERRULEKEY_ALARMFILTERRULEKEY,
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=570,
)


_ALARMFILTERRULE = _descriptor.Descriptor(
  name='AlarmFilterRule',
  full_name='voltha.AlarmFilterRule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='voltha.AlarmFilterRule.key', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='voltha.AlarmFilterRule.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=572,
  serialized_end=664,
)


_ALARMFILTER = _descriptor.Descriptor(
  name='AlarmFilter',
  full_name='voltha.AlarmFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='voltha.AlarmFilter.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='rules', full_name='voltha.AlarmFilter.rules', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=666,
  serialized_end=738,
)


_ALARMFILTERS = _descriptor.Descriptor(
  name='AlarmFilters',
  full_name='voltha.AlarmFilters',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filters', full_name='voltha.AlarmFilters.filters', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=740,
  serialized_end=792,
)


_LOGGING = _descriptor.Descriptor(
  name='Logging',
  full_name='voltha.Logging',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='level', full_name='voltha.Logging.level', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='package_name', full_name='voltha.Logging.package_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=794,
  serialized_end=867,
)


_COREINSTANCE = _descriptor.Descriptor(
  name='CoreInstance',
  full_name='voltha.CoreInstance',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_id', full_name='voltha.CoreInstance.instance_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='health', full_name='voltha.CoreInstance.health', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\000'))),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=869,
  serialized_end=963,
)


_COREINSTANCES = _descriptor.Descriptor(
  name='CoreInstances',
  full_name='voltha.CoreInstances',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='voltha.CoreInstances.items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=965,
  serialized_end=1024,
)


_VOLTHA = _descriptor.Descriptor(
  name='Voltha',
  full_name='voltha.Voltha',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='voltha.Voltha.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))),
    _descriptor.FieldDescriptor(
      name='adapters', full_name='voltha.Voltha.adapters', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='logical_devices', full_name='voltha.Voltha.logical_devices', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='devices', full_name='voltha.Voltha.devices', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='device_types', full_name='voltha.Voltha.device_types', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='device_groups', full_name='voltha.Voltha.device_groups', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='alarm_filters', full_name='voltha.Voltha.alarm_filters', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))),
    _descriptor.FieldDescriptor(
      name='omci_mib_database', full_name='voltha.Voltha.omci_mib_database', index=7,
      number=28, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))),
    _descriptor.FieldDescriptor(
      name='omci_alarm_database', full_name='voltha.Voltha.omci_alarm_database', index=8,
      number=29, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1027,
  serialized_end=1515,
)


_SELFTESTRESPONSE = _descriptor.Descriptor(
  name='SelfTestResponse',
  full_name='voltha.SelfTestResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='voltha.SelfTestResponse.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SELFTESTRESPONSE_SELFTESTRESULT,
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1518,
  serialized_end=1682,
)


_OFAGENTSUBSCRIBER = _descriptor.Descriptor(
  name='OfAgentSubscriber',
  full_name='voltha.OfAgentSubscriber',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ofagent_id', full_name='voltha.OfAgentSubscriber.ofagent_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='voltha_id', full_name='voltha.OfAgentSubscriber.voltha_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1684,
  serialized_end=1742,
)

_DEVICEGROUP.fields_by_name['logical_devices'].message_type = logical__device__pb2._LOGICALDEVICE
_DEVICEGROUP.fields_by_name['devices'].message_type = device__pb2._DEVICE
_DEVICEGROUPS.fields_by_name['items'].message_type = _DEVICEGROUP
_ALARMFILTERRULEKEY_ALARMFILTERRULEKEY.containing_type = _ALARMFILTERRULEKEY
_ALARMFILTERRULE.fields_by_name['key'].enum_type = _ALARMFILTERRULEKEY_ALARMFILTERRULEKEY
_ALARMFILTER.fields_by_name['rules'].message_type = _ALARMFILTERRULE
_ALARMFILTERS.fields_by_name['filters'].message_type = _ALARMFILTER
_LOGGING.fields_by_name['level'].enum_type = common__pb2._LOGLEVEL_LOGLEVEL
_COREINSTANCE.fields_by_name['health'].message_type = health__pb2._HEALTHSTATUS
_COREINSTANCES.fields_by_name['items'].message_type = _COREINSTANCE
_VOLTHA.fields_by_name['adapters'].message_type = adapter__pb2._ADAPTER
_VOLTHA.fields_by_name['logical_devices'].message_type = logical__device__pb2._LOGICALDEVICE
_VOLTHA.fields_by_name['devices'].message_type = device__pb2._DEVICE
_VOLTHA.fields_by_name['device_types'].message_type = device__pb2._DEVICETYPE
_VOLTHA.fields_by_name['device_groups'].message_type = _DEVICEGROUP
_VOLTHA.fields_by_name['alarm_filters'].message_type = _ALARMFILTER
_VOLTHA.fields_by_name['omci_mib_database'].message_type = omci__mib__db__pb2._MIBDEVICEDATA
_VOLTHA.fields_by_name['omci_alarm_database'].message_type = omci__alarm__db__pb2._ALARMDEVICEDATA
_SELFTESTRESPONSE.fields_by_name['result'].enum_type = _SELFTESTRESPONSE_SELFTESTRESULT
_SELFTESTRESPONSE_SELFTESTRESULT.containing_type = _SELFTESTRESPONSE
DESCRIPTOR.message_types_by_name['DeviceGroup'] = _DEVICEGROUP
DESCRIPTOR.message_types_by_name['DeviceGroups'] = _DEVICEGROUPS
DESCRIPTOR.message_types_by_name['AlarmFilterRuleKey'] = _ALARMFILTERRULEKEY
DESCRIPTOR.message_types_by_name['AlarmFilterRule'] = _ALARMFILTERRULE
DESCRIPTOR.message_types_by_name['AlarmFilter'] = _ALARMFILTER
DESCRIPTOR.message_types_by_name['AlarmFilters'] = _ALARMFILTERS
DESCRIPTOR.message_types_by_name['Logging'] = _LOGGING
DESCRIPTOR.message_types_by_name['CoreInstance'] = _COREINSTANCE
DESCRIPTOR.message_types_by_name['CoreInstances'] = _COREINSTANCES
DESCRIPTOR.message_types_by_name['Voltha'] = _VOLTHA
DESCRIPTOR.message_types_by_name['SelfTestResponse'] = _SELFTESTRESPONSE
DESCRIPTOR.message_types_by_name['OfAgentSubscriber'] = _OFAGENTSUBSCRIBER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeviceGroup = _reflection.GeneratedProtocolMessageType('DeviceGroup', (_message.Message,), dict(
  DESCRIPTOR = _DEVICEGROUP,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.DeviceGroup)
  ))
_sym_db.RegisterMessage(DeviceGroup)

DeviceGroups = _reflection.GeneratedProtocolMessageType('DeviceGroups', (_message.Message,), dict(
  DESCRIPTOR = _DEVICEGROUPS,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.DeviceGroups)
  ))
_sym_db.RegisterMessage(DeviceGroups)

AlarmFilterRuleKey = _reflection.GeneratedProtocolMessageType('AlarmFilterRuleKey', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTERRULEKEY,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilterRuleKey)
  ))
_sym_db.RegisterMessage(AlarmFilterRuleKey)

AlarmFilterRule = _reflection.GeneratedProtocolMessageType('AlarmFilterRule', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTERRULE,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilterRule)
  ))
_sym_db.RegisterMessage(AlarmFilterRule)

AlarmFilter = _reflection.GeneratedProtocolMessageType('AlarmFilter', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTER,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilter)
  ))
_sym_db.RegisterMessage(AlarmFilter)

AlarmFilters = _reflection.GeneratedProtocolMessageType('AlarmFilters', (_message.Message,), dict(
  DESCRIPTOR = _ALARMFILTERS,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.AlarmFilters)
  ))
_sym_db.RegisterMessage(AlarmFilters)

Logging = _reflection.GeneratedProtocolMessageType('Logging', (_message.Message,), dict(
  DESCRIPTOR = _LOGGING,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Logging)
  ))
_sym_db.RegisterMessage(Logging)

CoreInstance = _reflection.GeneratedProtocolMessageType('CoreInstance', (_message.Message,), dict(
  DESCRIPTOR = _COREINSTANCE,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.CoreInstance)
  ))
_sym_db.RegisterMessage(CoreInstance)

CoreInstances = _reflection.GeneratedProtocolMessageType('CoreInstances', (_message.Message,), dict(
  DESCRIPTOR = _COREINSTANCES,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.CoreInstances)
  ))
_sym_db.RegisterMessage(CoreInstances)

Voltha = _reflection.GeneratedProtocolMessageType('Voltha', (_message.Message,), dict(
  DESCRIPTOR = _VOLTHA,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.Voltha)
  ))
_sym_db.RegisterMessage(Voltha)

SelfTestResponse = _reflection.GeneratedProtocolMessageType('SelfTestResponse', (_message.Message,), dict(
  DESCRIPTOR = _SELFTESTRESPONSE,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.SelfTestResponse)
  ))
_sym_db.RegisterMessage(SelfTestResponse)

OfAgentSubscriber = _reflection.GeneratedProtocolMessageType('OfAgentSubscriber', (_message.Message,), dict(
  DESCRIPTOR = _OFAGENTSUBSCRIBER,
  __module__ = 'voltha_pb2'
  # @@protoc_insertion_point(class_scope:voltha.OfAgentSubscriber)
  ))
_sym_db.RegisterMessage(OfAgentSubscriber)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023org.opencord.volthaB\014VolthaProtosZ+github.com/opencord/voltha-go/protos/voltha\252\002\026Opencord.Voltha.Voltha'))
_DEVICEGROUP.fields_by_name['id'].has_options = True
_DEVICEGROUP.fields_by_name['id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_DEVICEGROUP.fields_by_name['logical_devices'].has_options = True
_DEVICEGROUP.fields_by_name['logical_devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_DEVICEGROUP.fields_by_name['devices'].has_options = True
_DEVICEGROUP.fields_by_name['devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_ALARMFILTERRULEKEY.has_options = True
_ALARMFILTERRULEKEY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000'))
_ALARMFILTER.fields_by_name['id'].has_options = True
_ALARMFILTER.fields_by_name['id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_COREINSTANCE.fields_by_name['instance_id'].has_options = True
_COREINSTANCE.fields_by_name['instance_id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_COREINSTANCE.fields_by_name['health'].has_options = True
_COREINSTANCE.fields_by_name['health']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\000'))
_COREINSTANCE.has_options = True
_COREINSTANCE._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001'))
_COREINSTANCES.has_options = True
_COREINSTANCES._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001'))
_VOLTHA.fields_by_name['version'].has_options = True
_VOLTHA.fields_by_name['version']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\350\366\315\035\001'))
_VOLTHA.fields_by_name['adapters'].has_options = True
_VOLTHA.fields_by_name['adapters']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['logical_devices'].has_options = True
_VOLTHA.fields_by_name['logical_devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['devices'].has_options = True
_VOLTHA.fields_by_name['devices']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['device_types'].has_options = True
_VOLTHA.fields_by_name['device_types']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['device_groups'].has_options = True
_VOLTHA.fields_by_name['device_groups']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['alarm_filters'].has_options = True
_VOLTHA.fields_by_name['alarm_filters']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\004\n\002id'))
_VOLTHA.fields_by_name['omci_mib_database'].has_options = True
_VOLTHA.fields_by_name['omci_mib_database']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))
_VOLTHA.fields_by_name['omci_alarm_database'].has_options = True
_VOLTHA.fields_by_name['omci_alarm_database']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\342\366\315\035\013\n\tdevice_id'))
_VOLTHA.has_options = True
_VOLTHA._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\370\366\315\035\001'))
_SELFTESTRESPONSE.has_options = True
_SELFTESTRESPONSE._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('\360\366\315\035\000'))
# @@protoc_insertion_point(module_scope)



#
# Copyright 2017 the original author or authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import structlog

from pyvoltha.protos import openflow_13_pb2 as ofp
from hashlib import md5

log = structlog.get_logger()

# aliases
ofb_field = ofp.ofp_oxm_ofb_field
action = ofp.ofp_action

# OFPAT_* shortcuts
OUTPUT = ofp.OFPAT_OUTPUT
COPY_TTL_OUT = ofp.OFPAT_COPY_TTL_OUT
COPY_TTL_IN = ofp.OFPAT_COPY_TTL_IN
SET_MPLS_TTL = ofp.OFPAT_SET_MPLS_TTL
DEC_MPLS_TTL = ofp.OFPAT_DEC_MPLS_TTL
PUSH_VLAN = ofp.OFPAT_PUSH_VLAN
POP_VLAN = ofp.OFPAT_POP_VLAN
PUSH_MPLS = ofp.OFPAT_PUSH_MPLS
POP_MPLS = ofp.OFPAT_POP_MPLS
SET_QUEUE = ofp.OFPAT_SET_QUEUE
GROUP = ofp.OFPAT_GROUP
SET_NW_TTL = ofp.OFPAT_SET_NW_TTL
NW_TTL = ofp.OFPAT_DEC_NW_TTL
SET_FIELD = ofp.OFPAT_SET_FIELD
PUSH_PBB = ofp.OFPAT_PUSH_PBB
POP_PBB = ofp.OFPAT_POP_PBB
EXPERIMENTER = ofp.OFPAT_EXPERIMENTER

# OFPXMT_OFB_* shortcuts (incomplete)
IN_PORT = ofp.OFPXMT_OFB_IN_PORT
IN_PHY_PORT = ofp.OFPXMT_OFB_IN_PHY_PORT
METADATA = ofp.OFPXMT_OFB_METADATA
ETH_DST = ofp.OFPXMT_OFB_ETH_DST
ETH_SRC = ofp.OFPXMT_OFB_ETH_SRC
ETH_TYPE = ofp.OFPXMT_OFB_ETH_TYPE
VLAN_VID = ofp.OFPXMT_OFB_VLAN_VID
VLAN_PCP = ofp.OFPXMT_OFB_VLAN_PCP
IP_DSCP = ofp.OFPXMT_OFB_IP_DSCP
IP_ECN = ofp.OFPXMT_OFB_IP_ECN
IP_PROTO = ofp.OFPXMT_OFB_IP_PROTO
IPV4_SRC = ofp.OFPXMT_OFB_IPV4_SRC
IPV4_DST = ofp.OFPXMT_OFB_IPV4_DST
TCP_SRC = ofp.OFPXMT_OFB_TCP_SRC
TCP_DST = ofp.OFPXMT_OFB_TCP_DST
UDP_SRC = ofp.OFPXMT_OFB_UDP_SRC
UDP_DST = ofp.OFPXMT_OFB_UDP_DST
SCTP_SRC = ofp.OFPXMT_OFB_SCTP_SRC
SCTP_DST = ofp.OFPXMT_OFB_SCTP_DST
ICMPV4_TYPE = ofp.OFPXMT_OFB_ICMPV4_TYPE
ICMPV4_CODE = ofp.OFPXMT_OFB_ICMPV4_CODE
ARP_OP = ofp.OFPXMT_OFB_ARP_OP
ARP_SPA = ofp.OFPXMT_OFB_ARP_SPA
ARP_TPA = ofp.OFPXMT_OFB_ARP_TPA
ARP_SHA = ofp.OFPXMT_OFB_ARP_SHA
ARP_THA = ofp.OFPXMT_OFB_ARP_THA
IPV6_SRC = ofp.OFPXMT_OFB_IPV6_SRC
IPV6_DST = ofp.OFPXMT_OFB_IPV6_DST
IPV6_FLABEL = ofp.OFPXMT_OFB_IPV6_FLABEL
ICMPV6_TYPE = ofp.OFPXMT_OFB_ICMPV6_TYPE
ICMPV6_CODE = ofp.OFPXMT_OFB_ICMPV6_CODE
IPV6_ND_TARGET = ofp.OFPXMT_OFB_IPV6_ND_TARGET
OFB_IPV6_ND_SLL = ofp.OFPXMT_OFB_IPV6_ND_SLL
IPV6_ND_TLL = ofp.OFPXMT_OFB_IPV6_ND_TLL
MPLS_LABEL = ofp.OFPXMT_OFB_MPLS_LABEL
MPLS_TC = ofp.OFPXMT_OFB_MPLS_TC
MPLS_BOS = ofp.OFPXMT_OFB_MPLS_BOS
PBB_ISID = ofp.OFPXMT_OFB_PBB_ISID
TUNNEL_ID = ofp.OFPXMT_OFB_TUNNEL_ID
IPV6_EXTHDR = ofp.OFPXMT_OFB_IPV6_EXTHDR


# ofp_action_* shortcuts

def output(port, max_len=ofp.OFPCML_MAX):
    return action(
        type=OUTPUT,
        output=ofp.ofp_action_output(port=port, max_len=max_len)
    )


def mpls_ttl(ttl):
    return action(
        type=SET_MPLS_TTL,
        mpls_ttl=ofp.ofp_action_mpls_ttl(mpls_ttl=ttl)
    )


def push_vlan(eth_type):
    return action(
        type=PUSH_VLAN,
        push=ofp.ofp_action_push(ethertype=eth_type)
    )


def pop_vlan():
    return action(
        type=POP_VLAN
    )


def pop_mpls(eth_type):
    return action(
        type=POP_MPLS,
        pop_mpls=ofp.ofp_action_pop_mpls(ethertype=eth_type)
    )


def group(group_id):
    return action(
        type=GROUP,
        group=ofp.ofp_action_group(group_id=group_id)
    )


def nw_ttl(nw_ttl):
    return action(
        type=NW_TTL,
        nw_ttl=ofp.ofp_action_nw_ttl(nw_ttl=nw_ttl)
    )


def set_field(field):
    return action(
        type=SET_FIELD,
        set_field=ofp.ofp_action_set_field(
            field=ofp.ofp_oxm_field(
                oxm_class=ofp.OFPXMC_OPENFLOW_BASIC,
                ofb_field=field))
    )


def experimenter(experimenter, data):
    return action(
        type=EXPERIMENTER,
        experimenter=ofp.ofp_action_experimenter(
            experimenter=experimenter, data=data)
    )


# ofb_field generators (incomplete set)

def in_port(_in_port):
    return ofb_field(type=IN_PORT, port=_in_port)


def in_phy_port(_in_phy_port):
    return ofb_field(type=IN_PHY_PORT, port=_in_phy_port)


def metadata(_table_metadata):
    return ofb_field(type=METADATA, table_metadata=_table_metadata)


def eth_dst(_eth_dst):
    return ofb_field(type=ETH_DST, table_metadata=_eth_dst)


def eth_src(_eth_src):
    return ofb_field(type=ETH_SRC, table_metadata=_eth_src)


def eth_type(_eth_type):
    return ofb_field(type=ETH_TYPE, eth_type=_eth_type)


def vlan_vid(_vlan_vid):
    return ofb_field(type=VLAN_VID, vlan_vid=_vlan_vid)


def vlan_pcp(_vlan_pcp):
    return ofb_field(type=VLAN_PCP, vlan_pcp=_vlan_pcp)


def ip_dscp(_ip_dscp):
    return ofb_field(type=IP_DSCP, ip_dscp=_ip_dscp)


def ip_ecn(_ip_ecn):
    return ofb_field(type=IP_ECN, ip_ecn=_ip_ecn)


def ip_proto(_ip_proto):
    return ofb_field(type=IP_PROTO, ip_proto=_ip_proto)


def ipv4_src(_ipv4_src):
    return ofb_field(type=IPV4_SRC, ipv4_src=_ipv4_src)


def ipv4_dst(_ipv4_dst):
    return ofb_field(type=IPV4_DST, ipv4_dst=_ipv4_dst)


def tcp_src(_tcp_src):
    return ofb_field(type=TCP_SRC, tcp_src=_tcp_src)


def tcp_dst(_tcp_dst):
    return ofb_field(type=TCP_DST, tcp_dst=_tcp_dst)


def udp_src(_udp_src):
    return ofb_field(type=UDP_SRC, udp_src=_udp_src)


def udp_dst(_udp_dst):
    return ofb_field(type=UDP_DST, udp_dst=_udp_dst)


def sctp_src(_sctp_src):
    return ofb_field(type=SCTP_SRC, sctp_src=_sctp_src)


def sctp_dst(_sctp_dst):
    return ofb_field(type=SCTP_DST, sctp_dst=_sctp_dst)


def icmpv4_type(_icmpv4_type):
    return ofb_field(type=ICMPV4_TYPE, icmpv4_type=_icmpv4_type)


def icmpv4_code(_icmpv4_code):
    return ofb_field(type=ICMPV4_CODE, icmpv4_code=_icmpv4_code)


def arp_op(_arp_op):
    return ofb_field(type=ARP_OP, arp_op=_arp_op)


def arp_spa(_arp_spa):
    return ofb_field(type=ARP_SPA, arp_spa=_arp_spa)


def arp_tpa(_arp_tpa):
    return ofb_field(type=ARP_TPA, arp_tpa=_arp_tpa)


def arp_sha(_arp_sha):
    return ofb_field(type=ARP_SHA, arp_sha=_arp_sha)


def arp_tha(_arp_tha):
    return ofb_field(type=ARP_THA, arp_tha=_arp_tha)


def ipv6_src(_ipv6_src):
    return ofb_field(type=IPV6_SRC, arp_tha=_ipv6_src)


def ipv6_dst(_ipv6_dst):
    return ofb_field(type=IPV6_DST, arp_tha=_ipv6_dst)


def ipv6_flabel(_ipv6_flabel):
    return ofb_field(type=IPV6_FLABEL, arp_tha=_ipv6_flabel)


def ipmpv6_type(_icmpv6_type):
    return ofb_field(type=ICMPV6_TYPE, arp_tha=_icmpv6_type)


def icmpv6_code(_icmpv6_code):
    return ofb_field(type=ICMPV6_CODE, arp_tha=_icmpv6_code)


def ipv6_nd_target(_ipv6_nd_target):
    return ofb_field(type=IPV6_ND_TARGET, arp_tha=_ipv6_nd_target)


def ofb_ipv6_nd_sll(_ofb_ipv6_nd_sll):
    return ofb_field(type=OFB_IPV6_ND_SLL, arp_tha=_ofb_ipv6_nd_sll)


def ipv6_nd_tll(_ipv6_nd_tll):
    return ofb_field(type=IPV6_ND_TLL, arp_tha=_ipv6_nd_tll)


def mpls_label(_mpls_label):
    return ofb_field(type=MPLS_LABEL, arp_tha=_mpls_label)


def mpls_tc(_mpls_tc):
    return ofb_field(type=MPLS_TC, arp_tha=_mpls_tc)


def mpls_bos(_mpls_bos):
    return ofb_field(type=MPLS_BOS, arp_tha=_mpls_bos)


def pbb_isid(_pbb_isid):
    return ofb_field(type=PBB_ISID, arp_tha=_pbb_isid)


def tunnel_id(_tunnel_id):
    return ofb_field(type=TUNNEL_ID, arp_tha=_tunnel_id)


def ipv6_exthdr(_ipv6_exthdr):
    return ofb_field(type=IPV6_EXTHDR, arp_tha=_ipv6_exthdr)


# frequently used extractors:

def get_actions(flow):
    """Extract list of ofp_action objects from flow spec object"""
    assert isinstance(flow, ofp.ofp_flow_stats)
    # we have the following hard assumptions for now
    for instruction in flow.instructions:
        if instruction.type == ofp.OFPIT_APPLY_ACTIONS:
            return instruction.actions.actions


def get_ofb_fields(flow):
    assert isinstance(flow, ofp.ofp_flow_stats)
    assert flow.match.type == ofp.OFPMT_OXM
    ofb_fields = []
    for field in flow.match.oxm_fields:
        assert field.oxm_class == ofp.OFPXMC_OPENFLOW_BASIC
        ofb_fields.append(field.ofb_field)
    return ofb_fields


def get_out_port(flow):
    for action in get_actions(flow):
        if action.type == OUTPUT:
            return action.output.port
    return None


def get_in_port(flow):
    for field in get_ofb_fields(flow):
        if field.type == IN_PORT:
            return field.port
    return None


def get_goto_table_id(flow):
    for instruction in flow.instructions:
        if instruction.type == ofp.OFPIT_GOTO_TABLE:
            return instruction.goto_table.table_id
    return None


def get_metadata(flow):
    ''' legacy get method (only want lower 32 bits '''
    for field in get_ofb_fields(flow):
        if field.type == METADATA:
            return field.table_metadata & 0xffffffff
    return None


def get_metadata_64_bit(flow):
    for field in get_ofb_fields(flow):
        if field.type == METADATA:
            return field.table_metadata
    return None


def get_port_number_from_metadata(flow):
    """
    The port number (UNI on ONU) is in the lower 32-bits of metadata and
    the inner_tag is in the upper 32-bits

    This is set in the ONOS OltPipeline as a metadata field
    """
    md = get_metadata_64_bit(flow)

    if md is None:
        return None

    if md <= 0xffffffff:
        log.warn('onos-upgrade-suggested',
                 netadata=md,
                 message='Legacy MetaData detected form OltPipeline')
        return md

    return md & 0xffffffff


def get_inner_tag_from_metadata(flow):
    """
    The port number (UNI on ONU) is in the lower 32-bits of metadata and
    the inner_tag is in the upper 32-bits

    This is set in the ONOS OltPipeline as a metadata field
    """
    md = get_metadata_64_bit(flow)

    if md is None:
        return None

    if md <= 0xffffffff:
        log.warn('onos-upgrade-suggested',
                 netadata=md,
                 message='Legacy MetaData detected form OltPipeline')
        return md

    return (md >> 32) & 0xffffffff


# test and extract next table and group information
def has_next_table(flow):
    return get_goto_table_id(flow) is not None


def get_group(flow):
    for action in get_actions(flow):
        if action.type == GROUP:
            return action.group.group_id
    return None


def has_group(flow):
    return get_group(flow) is not None


def mk_oxm_fields(match_fields):
    oxm_fields = [
        ofp.ofp_oxm_field(
            oxm_class=ofp.OFPXMC_OPENFLOW_BASIC,
            ofb_field=field
        ) for field in match_fields
    ]

    return oxm_fields


def mk_instructions_from_actions(actions):
    instructions_action = ofp.ofp_instruction_actions()
    instructions_action.actions.extend(actions)
    instruction = ofp.ofp_instruction(type=ofp.OFPIT_APPLY_ACTIONS,
                                      actions=instructions_action)
    return [instruction]


def mk_simple_flow_mod(match_fields, actions, command=ofp.OFPFC_ADD,
                       next_table_id=None, **kw):
    """
    Convenience function to generare ofp_flow_mod message with OXM BASIC match
    composed from the match_fields, and single APPLY_ACTIONS instruction with
    a list if ofp_action objects.
    :param match_fields: list(ofp_oxm_ofb_field)
    :param actions: list(ofp_action)
    :param command: one of OFPFC_*
    :param kw: additional keyword-based params to ofp_flow_mod
    :return: initialized ofp_flow_mod object
    """
    instructions = [
        ofp.ofp_instruction(
            type=ofp.OFPIT_APPLY_ACTIONS,
            actions=ofp.ofp_instruction_actions(actions=actions)
        )
    ]
    if next_table_id is not None:
        instructions.append(ofp.ofp_instruction(
            type=ofp.OFPIT_GOTO_TABLE,
            goto_table=ofp.ofp_instruction_goto_table(table_id=next_table_id)
        ))

    return ofp.ofp_flow_mod(
        command=command,
        match=ofp.ofp_match(
            type=ofp.OFPMT_OXM,
            oxm_fields=[
                ofp.ofp_oxm_field(
                    oxm_class=ofp.OFPXMC_OPENFLOW_BASIC,
                    ofb_field=field
                ) for field in match_fields
            ]
        ),
        instructions=instructions,
        **kw
    )


def mk_multicast_group_mod(group_id, buckets, command=ofp.OFPGC_ADD):
    group = ofp.ofp_group_mod(
        command=command,
        type=ofp.OFPGT_ALL,
        group_id=group_id,
        buckets=buckets
    )
    return group


def hash_flow_stats(flow):
    """
    Return unique 64-bit integer hash for flow covering the following
    attributes: 'table_id', 'priority', 'flags', 'cookie', 'match', '_instruction_string'
    """
    _instruction_string = ""
    for _instruction in flow.instructions:
        _instruction_string += _instruction.SerializeToString()

    hex = md5('{},{},{},{},{},{}'.format(
        flow.table_id,
        flow.priority,
        flow.flags,
        flow.cookie,
        flow.match.SerializeToString(),
        _instruction_string
    )).hexdigest()
    return int(hex[:16], 16)


def flow_stats_entry_from_flow_mod_message(mod):
    flow = ofp.ofp_flow_stats(
        table_id=mod.table_id,
        priority=mod.priority,
        idle_timeout=mod.idle_timeout,
        hard_timeout=mod.hard_timeout,
        flags=mod.flags,
        cookie=mod.cookie,
        match=mod.match,
        instructions=mod.instructions
    )
    flow.id = hash_flow_stats(flow)
    return flow


def group_entry_from_group_mod(mod):
    group = ofp.ofp_group_entry(
        desc=ofp.ofp_group_desc(
            type=mod.type,
            group_id=mod.group_id,
            buckets=mod.buckets
        ),
        stats=ofp.ofp_group_stats(
            group_id=mod.group_id
            # TODO do we need to instantiate bucket bins?
        )
    )
    return group


def mk_flow_stat(**kw):
    return flow_stats_entry_from_flow_mod_message(mk_simple_flow_mod(**kw))


def mk_group_stat(**kw):
    return group_entry_from_group_mod(mk_multicast_group_mod(**kw))



#
# Copyright 2018 the original author or authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from task import Task
from twisted.internet import reactor
from twisted.internet.defer import failure, inlineCallbacks, TimeoutError, returnValue
from pyvoltha.adapters.extensions.omci.omci_defs import ReasonCodes, EntityOperations
from pyvoltha.adapters.extensions.omci.omci_me import MEFrame
from pyvoltha.adapters.extensions.omci.omci_frame import OmciFrame
from pyvoltha.adapters.extensions.omci.omci_cc import DEFAULT_OMCI_TIMEOUT
from pyvoltha.adapters.extensions.omci.omci_messages import OmciGet
from pyvoltha.adapters.extensions.omci.omci_fields import OmciTableField

RC = ReasonCodes
OP = EntityOperations


class GetException(Exception):
    pass


class OmciGetRequest(Task):
    """
    OpenOMCI Get an OMCI ME Instance Attributes

    Upon completion, the Task deferred callback is invoked with a reference of
    this Task object.

    The Task has an initializer option (allow_failure) that will retry all
    requested attributes if the original request fails with a status code of
    9 (Attributes failed or unknown). This result means that an attribute
    is not supported by the ONU or that a mandatory/optional attribute could
    not be executed by the ONU, even if it is supported, for example,
    because of a range or type violation.
    """
    task_priority = 128
    name = "ONU OMCI Get Task"

    def __init__(self, omci_agent, device_id, entity_class, entity_id, attributes,
                 exclusive=False, allow_failure=False):
        """
        Class initialization

        :param omci_agent: (OmciAdapterAgent) OMCI Adapter agent
        :param device_id: (str) ONU Device ID
        :param entity_class: (EntityClass) ME Class to retrieve
        :param entity_id: (int) ME Class instance ID to retrieve
        :param attributes: (list or set) Name of attributes to retrieve
        :param exclusive: (bool) True if this GET request Task exclusively own the
                                 OMCI-CC while running. Default: False
        :param allow_failure: (bool) If true, attempt to get all valid attributes
                                     if the original request receives an error
                                     code of 9 (Attributes failed or unknown).
        """
        super(OmciGetRequest, self).__init__(OmciGetRequest.name,
                                             omci_agent,
                                             device_id,
                                             priority=OmciGetRequest.task_priority,
                                             exclusive=exclusive)
        self._device = omci_agent.get_device(device_id)
        self._entity_class = entity_class
        self._entity_id = entity_id
        self._attributes = attributes
        self._allow_failure = allow_failure
        self._failed_or_unknown_attributes = set()
        self._results = None
        self._local_deferred = None

    def cancel_deferred(self):
        super(OmciGetRequest, self).cancel_deferred()

        d, self._local_deferred = self._local_deferred, None
        try:
            if d is not None and not d.called:
                d.cancel()
        except:
            pass

    @property
    def me_class(self):
        """The OMCI Managed Entity Class associated with this request"""
        return self._entity_class

    @property
    def entity_id(self):
        """The ME Entity ID associated with this request"""
        return self._entity_id

    @property
    def attributes(self):
        """
        Return a dictionary of attributes for the request if the Get was
        successfully completed.  None otherwise
        """
        if self._results is None:
            return None

        omci_msg = self._results.fields['omci_message'].fields
        return omci_msg['data'] if 'data' in omci_msg else None

    @property
    def success_code(self):
        """
        Return the OMCI success/reason code for the Get Response.
        """
        if self._results is None:
            return None

        return self._results.fields['omci_message'].fields['success_code']

    @property
    def raw_results(self):
        """
        Return the raw Get Response OMCIFrame
        """
        return self._results

    def start(self):
        """
        Start MIB Capabilities task
        """
        super(OmciGetRequest, self).start()
        self._local_deferred = reactor.callLater(0, self.perform_get_omci)

    @property
    def failed_or_unknown_attributes(self):
        """
        Returns a set attributes that failed or unknown in the original get
        request that resulted in an initial status code of 9 (Attributes
        failed or unknown).

        :return: (set of str) attributes
        """
        return self._failed_or_unknown_attributes

    @inlineCallbacks
    def perform_get_omci(self):
        """
        Perform the initial get request
        """
        self.log.info('perform-get', entity_class=self._entity_class,
                      entity_id=self._entity_id, attributes=self._attributes)
        try:
            # If one or more attributes is a table attribute, get it separately
            def is_table_attr(attr):
                index = self._entity_class.attribute_name_to_index_map[attr]
                attr_def = self._entity_class.attributes[index]
                return isinstance(attr_def.field, OmciTableField)

            first_attributes = {attr for attr in self._attributes if not is_table_attr(attr)}
            table_attributes = {attr for attr in self._attributes if is_table_attr(attr)}

            frame = MEFrame(self._entity_class, self._entity_id, first_attributes).get()
            self.strobe_watchdog()
            results = yield self._device.omci_cc.send(frame)

            status = results.fields['omci_message'].fields['success_code']
            self.log.debug('perform-get-status', status=status)

            # Success?
            if status == RC.Success.value:
                self._results = results
                results_omci = results.fields['omci_message'].fields

                # Were all attributes fetched?
                missing_attr = frame.fields['omci_message'].fields['attributes_mask'] ^ \
                    results_omci['attributes_mask']

                if missing_attr > 0 or len(table_attributes) > 0:
                    self.log.info('perform-get-missing', num_missing=missing_attr,
                                  table_attr=table_attributes)
                    self.strobe_watchdog()
                    self._local_deferred = reactor.callLater(0,
                                                             self.perform_get_missing_attributes,
                                                             missing_attr,
                                                             table_attributes)
                    returnValue(self._local_deferred)

            elif status == RC.AttributeFailure.value:
                # What failed?  Note if only one attribute was attempted, then
                # that is an overall failure

                if not self._allow_failure or len(self._attributes) <= 1:
                    raise GetException('Get failed with status code: {}'.
                                       format(RC.AttributeFailure.value))

                self.strobe_watchdog()
                self._local_deferred = reactor.callLater(0,
                                                         self.perform_get_failed_attributes,
                                                         results,
                                                         self._attributes)
                returnValue(self._local_deferred)

            else:
                raise GetException('Get failed with status code: {}'.format(status))

            self.log.debug('get-completed')
            self.deferred.callback(self)

        except TimeoutError as e:
            self.deferred.errback(failure.Failure(e))

        except Exception as e:
            self.log.exception('perform-get', e=e, class_id=self._entity_class,
                               entity_id=self._entity_id, attributes=self._attributes)
            self.deferred.errback(failure.Failure(e))

    @inlineCallbacks
    def perform_get_missing_attributes(self, missing_attr, table_attributes):
        """
        This method is called when the original Get requests completes with success
        but not all attributes were returned.  This can happen if one or more of the
        attributes would have exceeded the space available in the OMCI frame.

        This routine iterates through the missing attributes and attempts to retrieve
        the ones that were missing.

        :param missing_attr: (int) Missing attributes bitmask
        :param table_attributes: (set) Attributes that need table get/get-next support
        """
        self.log.debug('perform-get-missing', attrs=missing_attr, tbl=table_attributes)

        # Retrieve missing attributes first (if any)
        results_omci = self._results.fields['omci_message'].fields

        for index in xrange(16):
            attr_mask = 1 << index

            if attr_mask & missing_attr:
                # Get this attribute
                frame = OmciFrame(
                    transaction_id=None,  # OMCI-CC will set
                    message_type=OmciGet.message_id,
                    omci_message=OmciGet(
                        entity_class=self._entity_class.class_id,
                        entity_id=self._entity_id,
                        attributes_mask=attr_mask
                    )
                )
                try:
                    self.strobe_watchdog()
                    get_results = yield self._device.omci_cc.send(frame)

                    get_omci = get_results.fields['omci_message'].fields
                    if get_omci['success_code'] != RC.Success.value:
                        continue

                    assert attr_mask == get_omci['attributes_mask'], 'wrong attribute'
                    results_omci['attributes_mask'] |= attr_mask

                    if results_omci.get('data') is None:
                        results_omci['data'] = dict()

                    results_omci['data'].update(get_omci['data'])

                except TimeoutError:
                    self.log.debug('missing-timeout')

                except Exception as e:
                    self.log.exception('missing-failure', e=e)

        # Now any table attributes. OMCI_CC handles background get/get-next sequencing
        for tbl_attr in table_attributes:
            attr_mask = self._entity_class.mask_for(tbl_attr)
            frame = OmciFrame(
                    transaction_id=None,  # OMCI-CC will set
                    message_type=OmciGet.message_id,
                    omci_message=OmciGet(
                            entity_class=self._entity_class.class_id,
                            entity_id=self._entity_id,
                            attributes_mask=attr_mask
                    )
            )
            try:
                timeout = 2 * DEFAULT_OMCI_TIMEOUT  # Multiple frames expected
                self.strobe_watchdog()
                get_results = yield self._device.omci_cc.send(frame,
                                                              timeout=timeout)
                self.strobe_watchdog()
                get_omci = get_results.fields['omci_message'].fields
                if get_omci['success_code'] != RC.Success.value:
                    continue

                if results_omci.get('data') is None:
                    results_omci['data'] = dict()

                results_omci['data'].update(get_omci['data'])

            except TimeoutError:
                self.log.debug('tbl-attr-timeout')

            except Exception as e:
                self.log.exception('tbl-attr-timeout', e=e)

        self.deferred.callback(self)

    @inlineCallbacks
    def perform_get_failed_attributes(self, tmp_results, attributes):
        """

        :param tmp_results:
        :param attributes:
        :return:
        """
        self.log.debug('perform-get-failed', attrs=attributes)

        for attr in attributes:
            try:
                frame = MEFrame(self._entity_class, self._entity_id, {attr}).get()

                self.strobe_watchdog()
                results = yield self._device.omci_cc.send(frame)

                status = results.fields['omci_message'].fields['success_code']

                if status == RC.AttributeFailure.value:
                    self.log.debug('unknown-or-invalid-attribute', attr=attr, status=status)
                    self._failed_or_unknown_attributes.add(attr)

                elif status != RC.Success.value:
                    self.log.warn('invalid-get', class_id=self._entity_class,
                                  attribute=attr, status=status)
                    self._failed_or_unknown_attributes.add(attr)

                else:
                    # Add to partial results and correct the status
                    tmp_results.fields['omci_message'].fields['success_code'] = status
                    tmp_results.fields['omci_message'].fields['attributes_mask'] |= \
                        results.fields['omci_message'].fields['attributes_mask']

                    if tmp_results.fields['omci_message'].fields.get('data') is None:
                        tmp_results.fields['omci_message'].fields['data'] = dict()

                    tmp_results.fields['omci_message'].fields['data'][attr] = \
                        results.fields['omci_message'].fields['data'][attr]

            except TimeoutError as e:
                self.log.debug('attr-timeout')

            except Exception as e:
                self.log.exception('attr-failure', e=e)

        self._results = tmp_results
        self.deferred.callback(self)



#
# Copyright 2017 the original author or authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""
# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path

# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='pyvoltha',
    version='0.1.7',
    description='VOLTHA Python support libraries',
    author='Chip Boling',
    author_email='chip@bcsw.net',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://wiki.opencord.org/display/CORD/VOLTHA',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
    packages=find_packages(exclude=['tests']),
    install_requires=[
        "argparse == 1.2.1",
        "arrow == 0.10.0",
        "bitstring == 3.1.5",
        "cmd2 == 0.7.0",
        "colorama == 0.3.9",
        "confluent-kafka == 0.11.5",
        "cython == 0.24.1",
        "decorator == 4.1.2",
        "docker-py == 1.10.6",
        "etcd3 == 0.7.0",
        "flake8 == 3.7.3",
        "fluent-logger == 0.6.0",
        "gevent == 1.4.0",
        "grpc == 0.3.post19",
        "grpcio == 1.3.5",
        "grpcio-tools == 1.3.5",
        "hash_ring == 1.3.1",
        "hexdump == 3.3",
        "jinja2 == 2.8",
        "jsonpatch == 1.16",
        "jsonschema == 2.6.0",
        "kafka_python == 1.3.5",
        "klein == 17.10.0",
        "kubernetes == 5.0.0",
        "netaddr == 0.7.19",
        "networkx == 2.0",
        "mock == 2.0.0",
        "nose == 1.3.7",
        "nose-exclude == 0.5.0",
        "nose-testconfig == 0.10",
        "netifaces == 0.10.6",
        "pcapy == 0.11.1",
        "pep8 == 1.7.1",
        "pep8-naming>=0.3.3",
        "protobuf == 3.3.0",
        "protobuf-to-dict == 0.1.0",
        "pyflakes == 2.1.0",
        "pylint == 1.9.4",
        "python-consul == 0.6.2",
        "pyOpenSSL == 17.3.0",
        "PyYAML == 3.12",
        "requests == 2.18.4",
        "scapy == 2.3.3",
        "service-identity == 17.0.0",
        "simplejson == 3.12.0",
        "six == 1.12.0",
        "structlog == 17.2.0",
        "termcolor == 1.1.0",
        "transitions == 0.6.4",
        "treq == 17.8.0",
        "Twisted == 17.9.0",
        "txaioetcd == 0.3.0",
        "urllib3 == 1.22",
        "afkak == 3.0.0.dev20181106",
    ],
    dependency_links=["git+https://github.com/ciena/afkak.git#egg=afkak-3.0.0.dev20181106"]
)
